﻿using System;
using System.Collections.Generic;

namespace _21_GenListeMitDelegates
{
    public class LinkedList<T>
    {
        public class LinkedListNode<T>
        {
            public T Value { get; set; }
            public LinkedListNode<T> Next { get; internal set; }

            public LinkedListNode(T value)
            {
                Value = value;
                Next = null;
            }
        }

        private LinkedListNode<T> head;
        private LinkedListNode<T> tail;

        public void Add(T value)
        {
            LinkedListNode<T> newNode = new LinkedListNode<T>(value);

            if (head == null)
            {
                head = newNode;
                tail = newNode;
            }
            else
            {
                tail.Next = newNode;
                tail = newNode;
            }
        }

        public void Print()
        {
            LinkedListNode<T> current = head;

            while (current != null)
            {
                Console.WriteLine(current.Value);
                current = current.Next;
            }
        }
        public IEnumerator<T> GetEnumerator()
        {
            LinkedListNode<T> current = head;

            while (current != null)
            {
                yield return current.Value;
                current = current.Next;
            }
        }
        public delegate T Func(T x);
        public void ForAll(Func f)
        {
            LinkedListNode<T> current = head;

            while (current != null)
            {
                Console.WriteLine(current.Value);
                current.Value = f(current.Value);
                current = current.Next;
            }
        }
        public delegate bool MyTest(T x);
        public void PrintFilter(MyTest testFkt)
        {
            LinkedListNode<T> current = head;

            while (current != null)
            {
                if (testFkt(current.Value)==true)
                    Console.WriteLine(current.Value);
                current = current.Next;
            }
            Console.WriteLine("---------------------");
        }
        //public IEnumerable<T> Filter(Func<T, bool> predicate)
        //public IEnumerable<T> Filter(MyTest predicate)

        public IEnumerable<T> Filter(Predicate<T> predicate)
        {
            LinkedListNode<T> current = head;
            while (current != null)
            {
                if (predicate(current.Value))
                {
                    yield return current.Value;
                }
                current = current.Next;
            }
        }
    }

    public class Person
    {
        public string Name { get; set; }
        public int Alter { get; set; }

        public Person(string name, int alter)
        {
            Name = name;
            Alter = alter;
        }
        public override string ToString() => Name;
    }

    class Program
    {
        static bool IstUngerade(int x)
        {
            return x % 2 != 0;
        }
        static void Main(string[] args)
        {
            LinkedList<int> list = new LinkedList<int>();
            list.Add(1);
            list.Add(2);
            list.Add(3);
            list.PrintFilter(IstUngerade);
            list.PrintFilter(x => x % 2 != 0);
            list.PrintFilter(x =>true);
            list.ForAll(x => x + 1);
            list.ForAll(delegate (int x) { return x + 2; });

            LinkedList<string> sl = new LinkedList<string>();
            sl.Add("Anton");
            sl.Add("Berta");
            sl.Add("Claudia");
            sl.Add("Dieter");
            sl.PrintFilter(x => x.Contains("er"));
            sl.PrintFilter(x => x[^1]=='a');

            LinkedList<Person> pl = new LinkedList<Person>();
            pl.Add(new Person("Max Mustermann", 30));
            pl.Add(new Person("Anna Müller", 25));
            pl.Add(new Person("John Smith", 35));
            pl.Add(new Person("Emma Johnson", 28));
            pl.Add(new Person("Michael Schmidt", 42));
            pl.PrintFilter(x => x.Name[0] == 'M');

            //foreach (var item in pl.Filter(x => x.Name.Contains("n")))
            foreach (var item in pl.Filter(x => x.Alter < 30))
                {
                    Console.WriteLine(item.Name);
            }

            foreach (var item in pl)
            {
                Console.WriteLine(item.Name); ;
            }
        }
    }
}
